/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>
#include <math.h>

/****************************************************************************
 *  Tests Boundary conditions with 2D and 3D ellipses                       *
 ***************************************************************************/
void draw_ellipse(Display          *dpy,
               XID              resourceID,
               PEXOCRequestType req_type)

  {
    PEXCoord    center;
    PEXCoord2D  center_2;

    PEXVector   major, minor;
    PEXVector2D major_2, minor_2;
    PEXColor    int_color, line_color, edge_color;
    double      radius;

    /*****************************************************************
     Top Row. 3D ellipses
    *****************************************************************/

    center.x = 0.20;
    center.y = 0.80;
    center.z = 0.0;

    major.x = 0.15;
    major.y = 0.0;
    major.z = 0.0;

    minor.x = 0.0;
    minor.y = 0.0;
    minor.z = 0.0;

    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeDashed);
    center.x = 0.5;
    major.x = 0.0;
    minor.y = 0.1;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    center.x = 0.85;
    minor.y = 0.0;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    /*****************************************************************
     Middle row. The behaviour is implementation dependant.
     Ellipses with axes non orthogonal. A circle with zero radius.
    *****************************************************************/
    center.x = 0.15;
    center.y = 0.5;
    major.x = 0.11;
    minor.x = 0.05;
    minor.y = 0.05;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    int_color.rgb.red = 0.0;             /* nose */
    int_color.rgb.green = 1.0;
    int_color.rgb.blue = 0.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &int_color);
    center.x = 0.5;
    center.y = 0.5;
    major.x = 0.0;
    major.y = 0.0;
    minor.x = 0.0;
    minor.y = 0.05;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    int_color.rgb.red = 1.0;              /* eyes */
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &int_color);
    center_2.x = 0.43;
    center_2.y = 0.55;
    radius = 0.0;
    PEXExtCircle2D(dpy, resourceID, req_type, &center_2, radius);

    center_2.x = 0.57;
    PEXExtCircle2D(dpy, resourceID, req_type, &center_2, radius);

    line_color.rgb.red = 1.0;             /* mouth */
    line_color.rgb.green = 0.0;
    line_color.rgb.blue = 0.0;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &line_color);
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeSolid);
    center_2.x = 0.5;
    center_2.y = 0.39;
    major_2.x = 0.06;
    major_2.y = 0.0;
    minor_2.x = 0.0;
    minor_2.y = 0.02;
    PEXExtEllipticalArc2D(dpy, resourceID, req_type, &center_2, &major_2,
      &minor_2, M_PI, 2.0*M_PI);
    
    int_color.rgb.blue = 1.0;                /* face */
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB,
      &int_color); 
    center_2.x = 0.5;
    center_2.y = 0.5;
    radius = 0.18;
    PEXExtCircle2D(dpy, resourceID, req_type, &center_2, radius);

    center_2.x = 0.85;
    major_2.x = 0.11;
    major_2.y = 0.0;
    minor_2.x = -0.05;
    minor_2.y = -0.05;
    PEXExtEllipse2D(dpy, resourceID, req_type, &center_2, &major_2, &minor_2);

    /*****************************************************************
     Bottom Row. 2D Ellipses
    *****************************************************************/
    
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleSolid);
    PEXSetSurfaceEdgeFlag(dpy, resourceID, req_type, PEXOn);
    center_2.x = 0.20;
    center_2.y = 0.20;
    major_2.x = 0.15;
    major_2.y = 0.0;
    minor_2.x = 0.0;
    minor_2.y = 0.0;
    PEXExtEllipse2D(dpy, resourceID, req_type, &center_2, &major_2, &minor_2);

    PEXSetSurfaceEdgeType(dpy, resourceID, req_type, PEXSurfaceEdgeDashed);
    center_2.x = 0.5;
    major_2.x = 0.0;
    minor_2.y = 0.1;
    PEXExtEllipse2D(dpy, resourceID, req_type, &center_2, &major_2, &minor_2);

    center_2.x = 0.75;
    minor_2.y = -0.1;
    edge_color.rgb.red = 1.0;
    edge_color.rgb.green = 0.0;
    edge_color.rgb.blue = 0.0;
    PEXSetSurfaceEdgeColor(dpy, resourceID, req_type,PEXColorTypeRGB, 
      &edge_color);
    PEXExtEllipse2D(dpy, resourceID, req_type, &center_2, &major_2, &minor_2);

    center_2.x = 0.85;
    minor_2.y = 0.0;
    PEXExtEllipse2D(dpy, resourceID, req_type, &center_2, &major_2, &minor_2);
  }

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


  {
    strcpy(test_name, "ellipse2");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "IBM Suspex Sample");
  }


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
  {
  }

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
  {

    describe_test( "\n" );
    describe_test( " Tests ellipses and circles with boundary conditions:\n");
    describe_test( "\n" );
    describe_test( " TOP ROW: 3 ellipses are drawn. 1st: minor axis = 0, should give a\n");
    describe_test( "          horizontal line.\n");
    describe_test( "          2nd: major axis = 0, should be rendered as a vertical line.\n");
    describe_test( "          3rd: Both major and minor axes are zero and only a point\n");
    describe_test( "          should be rendered.\n");
    describe_test( " MIDDLE ROW: The first and third columns have 3D and 2D ellipses with\n");
    describe_test( "             non-orthogonal axes. The results are implementation  dependent.\n");
    describe_test( "             In the center, two circles are rendered with both \n");
    describe_test( "             major and minor = 0; There should be two points. An ellipse with\n");
    describe_test( "             major = 0 which should give a vertical line. An elliptical arc 2D\n");
    describe_test( "             and a regular circle.\n");
    describe_test( " BOTTOM ROW: 2D ellipses: 1st Minor = 0; 2nd: Major = 0; 3rd: minor = -1.0,\n");
    describe_test( "             major = 0; 4th: minor,major = 0; Surface edge attributes\n");
    describe_test( "             are applied.\n");
    describe_test( "\n"); 

    draw_ellipse(dpy, resourceID, req_type);

  }

void testproc(Display *dpy, Window window, int cur_image)
  {
    image_testproc("ellipse2", "ellipse2", img_desc,
                    dpy,window, 0, 0,
                    glob_window_width, glob_window_height);
  }

