/*
******************************************************************************
** PEXlib Texture Mapping Test Code 
**
** $Source: /src/rcs/Cats/XSystem/pex/txm_utility_code/utils.c,v $
** $Revision: 510.1.200.3 $
** $Date: 94/03/08 17:12:24 $
** $Author: hmgr $
******************************************************************************
*/

/*
 *  THIS IS A PRELIMINARY VERSION!!
 */

/*
 * UTILS.C -- Holds utility functions.
 *            1- Opens a window and put textures in the window.
 *            2- Sets view point.
 *            3- Sets lighting.
 *            4- Error and Warning routines.
 *
 *            Designed by Bradley L. Saunders     Nov. 22, 1993
 *            Modified by Bradley L. Saunders     Dec. 17, 1993
 *            Modified by Bradley L. Saunders     Dec. 22, 1993
 */

/******************************************************************************/
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/******************************************************************************/


#include <stdio.h>
#include <math.h>
#include <values.h>
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXHPlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xlib.h>
#include <pexutcmap.h>		/* Color Utilities */
#include "param_struct.h"
#include "prototypes.h"


#define MAX_WINDOW_WIDTH 512
#define MAX_WINDOW_HEIGHT 1024 

void Error(char *err_str)
/*
 ******************************************************************************
 **    Description:
 **        This function is called whenever an error occurs.
 **
 **    Author:
 **       Brad Saunders                Oct. 18, 1993
 **
 **    Input parameters:
 **        char *err_str - an error message for the user.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        Any error. 
 **
 **    Exit Conditions:
 **        1) Function exits to shell.
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Nearly all functions. 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  fprintf(stdout, "ERROR: %s\n",err_str);
  exit(0);
}

void Warning(char *warn_str)
/*
 ******************************************************************************
 **    Description:
 **        This function is called whenever a warning occurs.
 **
 **    Author:
 **       Brad Saunders                Dec. 21, 1993
 **
 **    Input parameters:
 **        char *warn_str - a warning message for the user.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        Any warning. 
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Nearly all functions. 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  fprintf(stdout, "Warning: %s\n", warn_str);
}

void Copy_Data(char **data, 
               PEXEnumTypeIndex texel_type, 
               PEXExtTexelArray map,
               unsigned int *return_size)
/*
 ******************************************************************************
 **    Description:
 **        This function puts textures and alpha maps in to a char string
 **
 **    Author:
 **       Brad Saunders                Jan. 06, 1994
 **
 **    Input parameters:
 **       unsigned char *data - returns the map.
 **       PEXEnumTypeIndex texel_type - stores the texel type.
 **       PEXExtTexelArray map - the map to copy.
 **       unsigned int *return_size - to pass back the map size.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        When calling Put_Texture_In_Window.
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        Put_Texture_In_Window. 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  int i, j, index;
  unsigned long *long_data;
  unsigned long color;
  unsigned int size;

  int map_size;
  size = map.dimension.t0 * map.dimension.t1;
  *return_size = size;

  *data = (char *)malloc(sizeof(unsigned long) * size);
  long_data = (unsigned long *)*data;
  index = 0;
  for(i=map.dimension.t1-1; i>-1; i--)
  {
    for(j=0; j<map.dimension.t0; j++)
    {
      switch(texel_type)
      {
      case (PEXEnumTypeIndex)PEXExtTexelRGBFloat:
        color =
          (int)(map.array.rgb[i*map.dimension.t0+j].red * 255) << 16;
        color |=
          (int)(map.array.rgb[i*map.dimension.t0+j].green * 255) << 8;
        color |=
          (int)(map.array.rgb[i*map.dimension.t0+j].blue * 255);
      break;

      case (PEXEnumTypeIndex)PEXExtTexelRGBAlphaFloat:
        color =
          (int)(map.array.rgb_alpha[i*map.dimension.t0+j].red * 255) << 16;
        color |=
          (int)(map.array.rgb_alpha[i*map.dimension.t0+j].green * 255) << 8;
        color |=
          (int)(map.array.rgb_alpha[i*map.dimension.t0+j].blue * 255);
      break;

      case (PEXEnumTypeIndex)PEXExtTexelRGBInt8:
        color =
          map.array.rgb8[i*map.dimension.t0+j].red << 16;
        color |=
          map.array.rgb8[i*map.dimension.t0+j].green << 8;
        color |=
          map.array.rgb8[i*map.dimension.t0+j].blue;
      break;

      case (PEXEnumTypeIndex)PEXExtTexelRGBAlphaInt8:
        color =
          map.array.rgb_alpha8[i*map.dimension.t0+j].red << 16;
        color |=
          map.array.rgb_alpha8[i*map.dimension.t0+j].green << 8;
        color |=
          map.array.rgb_alpha8[i*map.dimension.t0+j].blue;
      break;
      }  /* End switch */

      long_data[index++] = color;
    }
  }
}


void Put_Texture_In_Window(Display *dpy, 
                           Input_Parameters *in, 
                           int map, 
                           int depth,
                           Window *rgbwin, 
                           Window *alphawin)
/*
 ******************************************************************************
 **    Description:
 **        This function puts textures and alpha maps in to windows
 **        so the CreateFilteredTMFromWindow can use them.
 **
 **    Author:
 **       Brad Saunders                Dec. 21, 1993
 **
 **    Input parameters:
 **       Display *dpy - A valid display.
 **       Input_Parameters *in - Holds the texture and alpha information.
 **       int map - an index telling which map to use.
 **       int depth - the depth of the window to open.
 **       Window *rgbwin - used to pass back the texture window pointer.
 **       Window *alphawin - used to pass back the alpha window pointer.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        When calling CreateFilteredTMFromWindow.
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        Error if unable to create windows.
 **
 **    Referenced by:
 **        main(). 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
 /*
  * These variables are used for opening up X windows 
  */
  Window window;
  Window awindow;
  PEXExtensionInfo *ext_info;
  char errString[PEXErrorStringLength];
  XVisualInfo vis_info;
  XStandardColormap cmap_info;
  PEXColorApproxEntry capx_info;
  int screen;
  XSizeHints hints;
  PEXUtVisualCriteria criteria;
  PEXUtWindowSpecification window_info;
  unsigned int unmet;
  Atom prop_atom;
  int result;
  XColor returned_background;

 /*
  * These variables are used for copying the texture into the window
  */
  XImage *ximage;
  Colormap colormap;
  XColor colors;
  unsigned long *mask1, *mask2;
  int i, j, index;
  char *data;
  unsigned long *long_data;
  unsigned long color;
  unsigned int size;
  PEXExtTexelArray *texture;
  GC gc;
  char errstr[ERRORSTRINGSIZE];
  int map_size;
  int found;
  unsigned long color_map[256];

  screen = DefaultScreen(dpy);
  hints.x = 0;
  hints.y = 0;
  hints.width = in->base_map[map].dimension.t0;
  if(hints.width > MAX_WINDOW_WIDTH)
  {
    hints.width = MAX_WINDOW_WIDTH;
    sprintf(errstr, 
      "Texture being clipped to width of %d",MAX_WINDOW_WIDTH);
    Warning(errstr);
  }

  hints.height = in->base_map[map].dimension.t1;
  if(hints.height > MAX_WINDOW_HEIGHT)
  {
    hints.height = MAX_WINDOW_HEIGHT;
    sprintf(errstr, 
      "Texture being clipped to height of %d",MAX_WINDOW_HEIGHT);
    Warning(errstr);
  }
  
  hints.flags = (USSize|USPosition);

  criteria.hard_criteria_mask = (PEXUtMinColors                 | 
				 PEXUtDepth                     | 0);
  criteria.soft_criteria_mask = (PEXUtStandardColormapProperty  | 0);
  criteria.min_colors = 2;
  criteria.standard_colormap_property = True;
  criteria.depth = depth;

  window_info.attr_mask = CWEventMask;
  window_info.attrs.event_mask = (ExposureMask         |
				  VisibilityChangeMask |
				  StructureNotifyMask  | 0);

  window_info.title = "Texture Map Window";
  window_info.size_hints = hints;
  window_info.parent = RootWindow (dpy, screen);
  window_info.border_width = 0;
  window_info.background_color_name = "black";
  window_info.border_color_name = "white";

  result = PEXUtCreateWindowAndColormap (dpy, 
                                         screen, 
                                         &criteria,
                                         &window_info, 
                                         &window,
                                         &vis_info, 
                                         &cmap_info, 
                                         &capx_info,
                                         &unmet, 
                                         &prop_atom,
                                         &returned_background);

  if (result != PEXUtSuccess)
  {
    static char *PEXUtFailStrings[] =
    {
      "PEXUtAllocFailure",          /* -4 maps to 0 */
      "PEXUtPEXFailure",	    /* -3 maps to 1 */
      "PEXUtXFailure",		    /* -2 maps to 2 */
      "PEXUtCriteriaFailure",	    /* -1 maps to 3 */
      "PEXUtSuccess",		    /*  0 maps to 4 */
      "PEXUtQualifiedSuccess",	    /* +1 maps to 5 */
    };

    sprintf(errstr,"PEXUtCreateWindowAndColormap() fails: %s\n",
            PEXUtFailStrings[ result + 4 ]);
    Error(errstr);
  }

 /*
  * Copy the texture data into a char string.
  */
  Copy_Data(&data,
            in->domain_data[map].data.color.texel_type, 
            in->base_map[map],
            &size);

  if(depth == 24)
  {
    ximage = XCreateImage(dpy, DefaultVisual(dpy, screen), vis_info.depth,
                          ZPixmap, 0, data, in->base_map[map].dimension.t0,
                          in->base_map[map].dimension.t1, 32, 0);
  }  
  else
  {
   /*
    * Extract the colormap from the data
    */
    long_data = (unsigned long *)data;
    /* Set to 1 to skip the first entry in the X colormap */
    map_size = 1;
    /* Assume that first entry in color map is black */
    color_map[0] = 0L;
    for(i=0; i<size; i++)
    {
      found = False;
      for(j=0; j<map_size; j++)
      {
        if(long_data[i] == color_map[j])
        {
          long_data[i] = j;
          found = True;
        }
      }
      if(!found)
      {
        color_map[map_size] = long_data[i];
        long_data[i] = map_size;
        map_size++;
        if(map_size == 256)
        {
          sprintf(errstr, "Texture %d has a colormap that is too large", map);
          Error(errstr);
        }
      }
    }

    for(i=0; i<size; i++)
    {
      data[i] = (unsigned char)long_data[i];
    } 

    colormap = XCreateColormap(dpy,
                               window,
                               vis_info.visual,
                               AllocAll);

    XInstallColormap(dpy,colormap);

    /* Allocate color cells; set color flags */
    XAllocColorCells(dpy, colormap,1,mask1,1,mask2,256);

    colors.flags= DoRed | DoGreen | DoBlue;

    for(i = 0; i < map_size; i++)
    {
      colors.pixel = i;
      colors.red = ((color_map[i] & 0xff0000) >> 16) * 256;
      colors.green = ((color_map[i] & 0xff00) >> 8) * 256; 
      colors.blue = (color_map[i] & 0xff) * 256;
      XStoreColor(dpy, colormap, &colors);
    }

    /* Set up the colormap */
    XSetWindowColormap(dpy, window, colormap);

    ximage = XCreateImage(dpy, DefaultVisual(dpy, screen), vis_info.depth,
                          ZPixmap, 0, data, in->base_map[map].dimension.t0,
                          in->base_map[map].dimension.t1, 8, 0);
  }  

  gc = XCreateGC(dpy, window, 0, (XGCValues *)NULL);
  XMapRaised(dpy, window);
  XFlush(dpy);

  XPutImage(dpy, window, gc, ximage, 0, 0, 0, 0, 
            in->base_map[map].dimension.t0,
            in->base_map[map].dimension.t1);
  
  *rgbwin = window;
  free(data);

 /*
  * If there is alpha data.
  * Note if you use one of the alpha structures this assumes that alpha
  * data is present.
  */
  if(in->domain_data[map].data.color.texel_type == 
    (PEXEnumTypeIndex)PEXExtTexelRGBAlphaFloat ||
     in->domain_data[map].data.color.texel_type ==
    (PEXEnumTypeIndex)PEXExtTexelRGBAlphaInt8)
  {
    hints.x = MAX_WINDOW_WIDTH + 30;
    hints.y = 0;
    hints.width = in->alpha_map[map].dimension.t0;
    if(hints.width > MAX_WINDOW_WIDTH)
    {
      hints.width = MAX_WINDOW_WIDTH;
      sprintf(errstr, 
        "Alpha map being clipped to width of %d",MAX_WINDOW_WIDTH);
      Warning(errstr);
    }
  
    hints.height = in->alpha_map[map].dimension.t1;
    if(hints.height > MAX_WINDOW_HEIGHT)
    {
      hints.height = MAX_WINDOW_HEIGHT;
      sprintf(errstr, 
        "Alpha map being clipped to height of %d",MAX_WINDOW_HEIGHT);
      Warning(errstr);
    }
  
    window_info.title = "Alpha Map Window";
    window_info.size_hints = hints;
  
    result = PEXUtCreateWindowAndColormap (dpy, 
                                           screen, 
                                           &criteria,
                                           &window_info, 
                                           &awindow,
                                           &vis_info, 
                                           &cmap_info, 
                                           &capx_info,
                                           &unmet, 
                                           &prop_atom,
                                           &returned_background);
  
    if (result != PEXUtSuccess)
    {
      static char *PEXUtFailStrings[] =
      {
        "PEXUtAllocFailure",          /* -4 maps to 0 */
        "PEXUtPEXFailure",	    /* -3 maps to 1 */
        "PEXUtXFailure",		    /* -2 maps to 2 */
        "PEXUtCriteriaFailure",	    /* -1 maps to 3 */
        "PEXUtSuccess",		    /*  0 maps to 4 */
        "PEXUtQualifiedSuccess",	    /* +1 maps to 5 */
      };
  
      sprintf(errstr,"PEXUtCreateWindowAndColormap() fails: %s\n",
              PEXUtFailStrings[ result + 4 ]);
      Error(errstr);
    }
  
   /*
    * Copy the texture data into a char string.
    */
    Copy_Data(&data,
              in->domain_data[map].data.color.texel_type, 
              in->alpha_map[map],
              &size);
  
    if(depth == 24)
    {
      ximage = XCreateImage(dpy, DefaultVisual(dpy, screen), vis_info.depth,
                            ZPixmap, 0, data, in->alpha_map[map].dimension.t0,
                            in->alpha_map[map].dimension.t1, 32, 0);
    }
    else
    {
      long_data = (unsigned long *)data;
      /* Set to 1 to skip the first entry in the X colormap */
      map_size = 1;
      for(i=0; i<size; i++)
      {
        found = False;
        for(j=0; j<map_size; j++)
        {
          if(long_data[i] == color_map[j])
          {
            long_data[i] = j+1;
            found = True;
          }
        }
        if(!found)
        {
          color_map[map_size] = long_data[i];
          long_data[i] = map_size;
          map_size++;
          if(map_size > 256)
          {
            sprintf(errstr, "Texture %d has a colormap that is too large", map);
            Error(errstr);
          }
        }
      }
  
      for(i=0; i<size; i++)
      {
        data[i] = long_data[i];
      }
  
      colormap = XCreateColormap(dpy,
                                 awindow,
                                 vis_info.visual,
                                 AllocAll);
  
      XInstallColormap(dpy,colormap);
  
      /* Allocate color cells; set color flags */
      XAllocColorCells(dpy, colormap,1,mask1,1,mask2,256);
  
      colors.flags= DoRed | DoGreen | DoBlue;
  
      for(i = 0; i < map_size; i++)
      {
        colors.pixel = i+1;
        colors.red = ((color_map[i] & 0xff0000) >> 16) * 256;
        colors.green = ((color_map[i] & 0xff00) >> 8) * 256;
        colors.blue = (color_map[i] & 0xff) * 256;
        XStoreColor(dpy, colormap, &colors);
      }
  
      /* Set up the colormap */
      XSetWindowColormap(dpy, awindow, colormap);
  
      ximage = XCreateImage(dpy, DefaultVisual(dpy, screen), vis_info.depth,
                            ZPixmap, 0, data, in->alpha_map[map].dimension.t0,
                            in->alpha_map[map].dimension.t1, 8, 0);
    }
  
    gc = XCreateGC(dpy, awindow, 0, (XGCValues *)NULL);
    XMapRaised(dpy, awindow);
    XFlush(dpy);
  
    XPutImage(dpy, awindow, gc, ximage, 0, 0, 0, 0,
              in->alpha_map[map].dimension.t0,
              in->alpha_map[map].dimension.t1);
  
    *alphawin = awindow;
    free(data);
  }
  else
    *alphawin = (Window)NULL;
}
  
void set_view(Display *dpy,
              PEXLookupTable view_table,
              Input_Parameters in)
/*
 ******************************************************************************
 **    Description:
 **        This function sets a custom view point.
 **
 **    Author:
 **       Brad Saunders                Dec. 21, 1993
 **
 **    Input parameters:
 **       Display *dpy - a valid display.
 **       PEXLookupTable view_table - a valid pex view table.
 **       Input_Parameters in - stores the view data.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        If custom view point is requested. 
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        main(). 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
 /*
  * Compute the view orientation transform.
  */
    (void)PEXViewOrientationMatrix( &in.view_ref_pt, &in.view_plane_normal,
                                    &in.view_up_vec, in.view.orientation );

 /*
  * Compute the view mapping transform.
  */
    (void)PEXViewMappingMatrix(in.window,
                               &in.viewport,
                               in.do_perspective,
                               &in.prp,
                               in.view_plane,
                               in.back_plane,
                               in.front_plane,
                               in.view.mapping);

 /*
  * Set view
  */
    PEXSetTableEntries(dpy,
                       view_table,
                       1,
                       1,
                       PEXLUTView,
                       (PEXPointer) &in.view );
}

void normalize(PEXVector *vector)
/*
 ******************************************************************************
 **    Description:
 **        This function normailizes a vector.
 **
 **    Author:
 **       Brad Saunders()                Dec. 21, 1993
 **
 **    Input parameters:
 **       PEXVector *vector - a vector.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        When a vector needs to be normalized for lighting. 
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        set_lights(). 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  float r;

  /* 
   * Transform a vector to a unit vector. 
   */
  r = vector->x * vector->x +
      vector->y * vector->y +
      vector->z * vector->z;
  if (r > 0) 
  {
    r = 1.0 / sqrt(r);
    vector->x *= r;
    vector->y *= r;
    vector->z *= r;
  }
}

void set_lights(Display *dpy,
                PEXLookupTable light_table,
                Input_Parameters in,
                int *num_lights)
/*
 ******************************************************************************
 **    Description:
 **        This function sets custom lighting.
 **
 **    Author:
 **       Brad Saunders                Dec. 21, 1993
 **
 **    Input parameters:
 **       Display *dpy - a valid display.
 **       PEXLookupTable light_table - a valid pex light table.
 **       Input_Parameters in - stores the lighting index.
 **       int *num_lights - a pointer to return the number of lights turned on.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        If custom lights are requested. 
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        main(). 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  PEXLightEntry       lights[4];
  char errstr[ERRORSTRINGSIZE];

  switch(in.light_index)
  {
  case 1:
    /* Define the ambient light. */
    lights[0].type = PEXLightAmbient;
    lights[0].color.type = PEXColorTypeRGB;
    lights[0].color.value.rgb.red = .3;
    lights[0].color.value.rgb.green = .3;
    lights[0].color.value.rgb.blue = .3;

    /* Define the directional light. */
    lights[1].type = PEXLightWCVector;
    lights[1].direction.x = -1;
    lights[1].direction.y = -1;
    lights[1].direction.z = 1;
    lights[1].color.type = PEXColorTypeRGB;
    lights[1].color.value.rgb.red = 1.0;
    lights[1].color.value.rgb.green = 1.0;
    lights[1].color.value.rgb.blue = 1.0;

    /* Define the point light. */
    lights[2].type = PEXLightWCPoint;
    lights[2].point.x = -100.0;
    lights[2].point.y = 100.0;
    lights[2].point.z = 100.0;
    lights[2].attenuation1 = 1;
    lights[2].attenuation2 = 1;
    lights[2].color.type = PEXColorTypeRGB;
    lights[2].color.value.rgb.red = 1.0;
    lights[2].color.value.rgb.green = 1.0;
    lights[2].color.value.rgb.blue = 1.0;

    /* Define the spot light. */
    lights[3].type = PEXLightWCSpot;
    lights[3].point.x = 1;
    lights[3].point.y = 1;
    lights[3].point.z = 1;
    lights[3].direction.x = -1;
    lights[3].direction.y = -1;
    lights[3].direction.z = -1;
    normalize( &lights[3].direction );
    lights[3].concentration = 75;
    lights[3].spread_angle = M_PI/4;
    lights[3].attenuation1 = 1;
    lights[3].attenuation2 = 0.2;
    lights[3].color.type = PEXColorTypeRGB;
    lights[3].color.value.rgb.red = 0.7;
    lights[3].color.value.rgb.green = 0.7;
    lights[3].color.value.rgb.blue = 0.7;

    /* Add the lights to the light table. */
    PEXSetTableEntries(dpy, light_table, 1, 4, PEXLUTLight, lights);
    *num_lights = 4;    
  break;

  case 2:
    /* Define the ambient light. */
    lights[0].type = PEXLightAmbient;
    lights[0].color.type = PEXColorTypeRGB;
    lights[0].color.value.rgb.red = .3;
    lights[0].color.value.rgb.green = .3;
    lights[0].color.value.rgb.blue = .3;

    /* Define the directional light. */
    lights[1].type = PEXLightWCVector;
    lights[1].direction.x = -1;
    lights[1].direction.y = -1;
    lights[1].direction.z = 1;
    lights[1].color.type = PEXColorTypeRGB;
    lights[1].color.value.rgb.red = 0.5;
    lights[1].color.value.rgb.green = 0.5;
    lights[1].color.value.rgb.blue = 0.5;

    /* Define the point light. */
    lights[2].type = PEXLightWCPoint;
    lights[2].point.x = -100.0;
    lights[2].point.y = 100.0;
    lights[2].point.z = 100.0;
    lights[2].attenuation1 = .5;
    lights[2].attenuation2 = .5;
    lights[2].color.type = PEXColorTypeRGB;
    lights[2].color.value.rgb.red = 1.0;
    lights[2].color.value.rgb.green = 1.0;
    lights[2].color.value.rgb.blue = 1.0;

    /* Define the spot light. */
    lights[3].type = PEXLightWCSpot;
    lights[3].point.x = 1;
    lights[3].point.y = 1;
    lights[3].point.z = 1;
    lights[3].direction.x = -1;
    lights[3].direction.y = -1;
    lights[3].direction.z = -1;
    normalize( &lights[3].direction );
    lights[3].concentration = 25;
    lights[3].spread_angle = M_PI/4;
    lights[3].attenuation1 = 1;
    lights[3].attenuation2 = 0.2;
    lights[3].color.type = PEXColorTypeRGB;
    lights[3].color.value.rgb.red = 0.7;
    lights[3].color.value.rgb.green = 0.7;
    lights[3].color.value.rgb.blue = 0.7;

    /* Add the lights to the light table. */
    PEXSetTableEntries(dpy, light_table, 1, 4, PEXLUTLight, lights);
    *num_lights = 4;    
  break;

  default:
    sprintf(errstr, "Index %d, is not a valid lighting index",in.light_index);
    Warning(errstr);
    *num_lights = 0;
  break;
  }
}

void set_depth_cue(Display *dpy,
                   PEXLookupTable depth_cue_table,
                   Input_Parameters in)
/*
 ******************************************************************************
 **    Description:
 **        This function sets custom depth cueing.
 **
 **    Author:
 **       Brad Saunders                Dec. 21, 1993
 **
 **    Input parameters:
 **       Display *dpy - a valid display.
 **       PEXLookupTable depth_cue_table - a valid pex depth cue table.
 **       Input_Parameters in - stores the depth cue data.
 **
 **    Output parameters:
 **        None.
 **
 **    Entry Conditions:
 **        If depth cueing is requested. 
 **
 **    Exit Conditions:
 **        None. 
 **
 **    Error Conditions:
 **        None.
 **
 **    Referenced by:
 **        main(). 
 **
 **    Procedure/Function dependencies:
 **        None.
 **
 ******************************************************************************
 */
{
  PEXSetTableEntries(dpy, depth_cue_table, 1, 1, PEXLUTDepthCue, &in.dcue);
}
